/*
	@file
	irsend.c
	Version v.1.0 24.10.2016
	by gh0stless for www.crazy-midi.de
*/

#include "ext.h"			// you must include this - it contains the external object's link to available Max functions
#include "ext_obex.h"		// this is required for all objects using the newer style for writing objects.

typedef struct _irsend {	// defines our object's internal variables for each instance in a patch
	t_object p_ob;			// object header - ALL objects MUST begin with this...
	t_symbol *r_name;		
	void *p_outlet;			// outlet creation - inlets are automatic, but objects must "own" their own outlets
	void *m_qelem;
	char s_string[127];
} t_irsend;

// these are prototypes for the methods that are defined below
void irsend_sy1(t_irsend *x, t_symbol *s, long argc, t_atom *argv);
void irsend_assist(t_irsend *x, void *b, long m, long a, char *s);
void *irsend_new(long n);
void irsend_free(t_irsend *x);
void irsend_qtask(t_irsend *x);
void irsend_anything(t_irsend *x, t_symbol *s, long argc, t_atom *argv);

t_class *irsend_class;		// global pointer to the object class - so max can reference the object

//--------------------------------------------------------------------------

void ext_main(void *r)
{
	t_class *c;

	c = class_new("irsend", (method)irsend_new, (method)irsend_free, sizeof(t_irsend), 0L, A_DEFLONG, 0);
	// class_new() loads our external's class into Max's memory so it can be used in a patch
	// irsend_new = object creation method defined below
	class_addmethod(c, (method)irsend_sy1, "send", A_GIMME, 0);
	class_addmethod(c, (method)irsend_anything, "anything", A_GIMME, 0);
	class_addmethod(c, (method)irsend_assist,	"assist",	A_CANT, 0);	// (optional) assistance method needs to be declared like this
	class_register(CLASS_BOX, c);
	irsend_class = c;
	post("irsend object loaded...",0);	// post any important info to the max window when our class is loaded
}

//--------------------------------------------------------------------------

void *irsend_new(long n)		// n = int argument typed into object box (A_DEFLONG) -- defaults to 0 if no args are typed
{
	t_irsend *x;				// local variable (pointer to a t_irsend data structure)
	x = (t_irsend *)object_alloc(irsend_class); // create a new instance of this object
	x->p_outlet = bangout(x);	// create an bang outlet
	x->r_name	= NULL;			// set initial (default) right operand value (n = variable passed to irsend_new)
	x->m_qelem = qelem_new((t_object *)x, (method)irsend_qtask);
	post(" new irsend object instance added to patch...",0); // post important info to the max window when new instance is created
	return(x);					// return a reference to the object instance
}

void irsend_free(t_irsend *x)

{
	if (x->m_qelem)
		qelem_free(x->m_qelem);
}


//--------------------------------------------------------------------------

void irsend_assist(t_irsend *x, void *b, long m, long a, char *s) // 4 final arguments are always the same for the assistance method
{
	if (m == ASSIST_OUTLET)
		sprintf(s,"bang, if send");
	else {
		switch (a) {
		case 0:
			sprintf(s,"send IR-Name Code-Name Count");
			break;
		}
	}
}

void irsend_sy1(t_irsend *x, t_symbol *s, long argc, t_atom *argv)
{
	t_atom *ap;
 	ap = argv;
	if (argc == 3) {
		if ((atom_gettype(ap++) == A_SYM) && (atom_gettype(ap++) == A_SYM) && (atom_gettype(ap++) == A_LONG)) {
			char str1[127];
			char hstr[7];
			strcpy(str1, atom_getsym(argv)->s_name);
			strcat(str1, " ");
			strcat(str1, atom_getsym(argv + 1)->s_name);
			strcat(str1, " ");
			int aInt = atom_getlong(argv + 2);
			sprintf(hstr, "%i", aInt);
			strcat(str1, hstr);
			strcpy(x->s_string, str1);
			qelem_set(x->m_qelem);
		}
		else {
			post("irsend: Error! Wrong Parameters!");
		}
	}
	else {
		post("irsend: Error! Not 3 Parameters!");
	}
}

void irsend_qtask(t_irsend *x)
{
		LRESULT copyDataResult;
		HWND pOtherWnd = FindWindow(NULL, "WinLirc");
		if (pOtherWnd)
		{
			COPYDATASTRUCT cpd;
			cpd.dwData = 0;
			cpd.cbData = strlen(x->s_string) + 1;
			cpd.lpData = (void*)x->s_string;
			copyDataResult = SendMessage(pOtherWnd, WM_COPYDATA, (WPARAM)(HINSTANCE)GetModuleHandle(NULL), (LPARAM)&cpd);
			//post("irsend: I send: %s", x->s_string);
			outlet_bang(x->p_outlet); //Send a Bang
		}
		else
		{
			post("irsend: Error! WinLIRC not running?"); //Error
		}
}
void irsend_anything(t_irsend *x, t_symbol *s, long argc, t_atom *argv)
{
	post("irsend: Error! Wrong Message!");
}
